import React from 'react'
import {
	Animated,
	PanResponder,
	ViewPropTypes,
	Dimensions
} from 'react-native'
import PropTypes from "prop-types";

const {width, height} = Dimensions.get('window');

class CircleDrag extends React.Component{

	static propTypes = {
		style: ViewPropTypes.style,
		children: PropTypes.node,
		onPress: PropTypes.func,
		circleWidth: PropTypes.number,
		circleX: PropTypes.number,
		circleY: PropTypes.number,
	};

	static defaultProps = {
		circleWidth: 45,
		circleX: width - 45 - 10,
		circleY: height - 100 - 45
	};

	constructor(props) {
		super(props);
		this.state ={
			circle: props.circleWidth,
			trans: new Animated.ValueXY({x: props.circleX, y: props.circleY}),
		};
		this.pressStatus = false;
	}

	componentWillMount() {
		this._panResponder = PanResponder.create({
			onStartShouldSetPanResponder: () => true, //响应手势
			onStartShouldSetPanResponderCapture: (e, gestureState) => true,
			//开启移动手势响应
			onMoveShouldSetPanResponder: (evt, gestureState) => true,
			//开启移动手势响应是否劫持 true：不传递给子view false：传递给子view
			onMoveShouldSetPanResponderCapture: (evt, gestureState) => true,
			onPanResponderGrant: this._onPanResponderGrant,
			onPanResponderMove: this._onPanResponderMove,
			onPanResponderRelease: this._onPanResponderRelease,
			onPanResponderTerminationRequest: (evt, gestureState) => true,
		});
	}

	_onPanResponderGrant = (evt, gestureState) => {
		this.pressStatus = true;
		// 开始手势操作。
		//将trans初始值设置为起始点
		this.state.trans.setOffset({
			x: this.state.trans.x._value,
			y: this.state.trans.y._value,
		});
		this.state.trans.setValue({x: 0, y: 0});
	};

	_onPanResponderMove = (evt, gestureState) => {
		Animated.event([
			null,
			{dx: this.state.trans.x, dy:this.state.trans.y},
		])(evt, gestureState);

		if (gestureState.dx < 5 && gestureState.dx > -5 && gestureState.dy < 5 && gestureState.dy > -5){
			this.pressStatus = true;
		} else {
			this.pressStatus = false;
		}
	};

	_onPanResponderRelease = (evt, gestureState) => {
		let circleW= this.props.circleWidth;
		if (this.pressStatus) {
			//点击手势
			this.props.onPress && this.props.onPress();
			this.pressStatus = false;
		} else {
			//拖拽手势
			let tabH = ISIPHONEX() ? 83 : 50;
			let statusBarH = ISIPHONEX() ? 44 : 25;
			if (gestureState.moveX <= circleW && gestureState.moveY <=circleW) {
				this.state.trans.setOffset({x: 0, y: 0});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveX >= (width - circleW) && gestureState.moveY <=circleW) {
				this.state.trans.setOffset({x: width - circleW, y: 0});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveX <= circleW && gestureState.moveY >= (height - tabH - circleW - statusBarH)) {
				this.state.trans.setOffset({x: 0, y: (height - tabH - circleW - statusBarH)});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveX >= (width - circleW) && gestureState.moveY >= (height - tabH - circleW - statusBarH)) {
				this.state.trans.setOffset({x: width - circleW, y: (height - tabH - circleW - statusBarH)});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveX <= circleW) {
				this.state.trans.setOffset({x: 0, y: gestureState.moveY - circleW});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveX >= width - circleW) {
				this.state.trans.setOffset({x: width - circleW, y: gestureState.moveY - circleW});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveY >= height - tabH - circleW) {
				this.state.trans.setOffset({x: gestureState.moveX - 20, y: height - tabH - circleW - statusBarH});
				this.state.trans.setValue({x: 0, y: 0});
			} else if (gestureState.moveY <= 75) {
				this.state.trans.setOffset({x: gestureState.moveX - 20, y: 0});
				this.state.trans.setValue({x: 0, y: 0});
			}
		}
		this.state.trans.flattenOffset();
	}

	render() {
		return (
			<Animated.View
				collapsable={false}
				ref={'overFlowView'}
				style={[{
					justifyContent: 'center',
					alignItems: 'center',
					width: this.props.circleWidth,height: this.props.circleWidth,borderRadius: this.props.circleWidth/2,position: 'absolute',
					transform:[ {translateY:this.state.trans.y}, {translateX:this.state.trans.x}]},this.props.style]}
				{...this._panResponder.panHandlers}
			>
				{this.props.children}
			</Animated.View>
		)
	}
}

export default CircleDrag;
